tool
extends Node2D
class_name Unit

export var unit_type : String
export var max_health : int = 10
export var health : int = 10
export var movement_distance : int = 10
export var height : int = 0
export var water_walk : bool = false
export var air_walk : bool = false
export var lava_walk : bool = false
export var is_behind : bool = false
export var sprite : Texture setget set_sprite

var _target_position : Vector2

var movement_points : int = 0
var attack_points : int = 0

var _has_sent_turn_completed : bool = true

signal turn_completed

onready var Grid = get_node("../Grid")


func start_turn():
	attack_points = 0
	movement_points = movement_distance
	_has_sent_turn_completed = false


func set_sprite(value: Texture) -> void:
	sprite = value
	var _sprite = $Sprite
	# Setter functions are called during the node's `_init()` callback, before they entered the
	# tree. At that point in time, the `_sprite` variable is `null`. If so, we have to wait to
	# update the sprite's properties.
	if not _sprite:
		# The yield keyword allows us to wait until the unit node's `_ready()` callback ended.
		yield(self, "ready")
	_sprite.texture = value


func _ready() -> void:
	#registers itself on the grid and snaps to a tile
	yield(get_tree().create_timer(.01),"timeout")
	position = Grid.register_new_unit(self)
	
	z_index = height * 10 + 5
	if not is_behind:
		z_index = 100


func move(movement : Vector2):
	if movement_points != 0: #checks if the unit has had its turn yet
		
		var potential_movement = Grid.move_request(movement, self) #asks the grid if it can move
		if potential_movement: #if not null, and therefore the grid returned a new position
			
			_target_position = potential_movement[0]
			var _target_height = potential_movement[1]
			
			#take away the movement points depending on which radius zone the unit is in
			#tries radius 0, then 1, the 2, etc. until the unit is in that radius and takes it away
			for i in range(movement_points, -1, -1):
				var area_around = Grid.get_around_coordinate(Grid.position_to_coordinates(position, height), i)
				if not area_around.has(Grid.position_to_coordinates(_target_position, _target_height)):
					movement_points = movement_points - i - 1
					print(movement_points)
					break
			
			#change the z_index
			height = potential_movement[1]
			z_index = height
			if (potential_movement[2] == false): #not behind
				z_index = 100


func attack(): #TODO
	if attack_points == 0:
		#change animation to attacking
		#make attacked change to hurt animation
		#somehow change health values
		#maybe using an attacked method??
		attack_points = 0


func _physics_process(delta: float) -> void:
	if _target_position: #TODO figure out better movement system (pathfinding)
		position.x = move_toward(position.x, _target_position.x, 800 * delta)
		position.y = move_toward(position.y, _target_position.y, 800 * delta)
	
	attack_points = false
	
	if attack_points == 0 && movement_points == 0 && not _has_sent_turn_completed:
		emit_signal("turn_completed")
		_has_sent_turn_completed = true

